const Main = imports.ui.main;
const Lang = imports.lang;
const GLib = imports.gi.GLib;
const PointerWatcher = imports.ui.pointerWatcher;
const AppDisplay = imports.ui.appDisplay;

const MOUSE_POLL_FREQUENCY = 10;
const CURSOR_UPDATE_TIMEOUT = 100;

let injections = [];
let handlers = [];

function enable() {
	injections['addWatch'] = PointerWatcher.PointerWatcher.prototype.addWatch;
	PointerWatcher.PointerWatcher.prototype.addWatch = function (interval, callback) {
		return injections['addWatch'].call(this, MOUSE_POLL_FREQUENCY, callback);
	}

	injections['adaptToSize'] = AppDisplay.AllView.prototype.adaptToSize;
	AppDisplay.AllView.prototype.adaptToSize = function (width, height) {
		injections['adaptToSize'].call(this, width, height);
		if (Main.magnifier.isActive())
			this._scrollView.update_fade_effect(0, 0);
	}

	handlers[handlers.length] = Main.magnifier.connect('active-changed', Lang.bind(Main.magnifier, function () {
		if (this.isActive() && !this._cursorTimeoutId) {
			this._cursorTimeoutId = GLib.timeout_add(GLib.PRIORITY_DEFAULT, CURSOR_UPDATE_TIMEOUT, Lang.bind(this, function () {
					this._updateMouseSprite();
					return true;
				}));
		} else if (!this.isActive() && this._cursorTimeoutId) {
			GLib.source_remove(this._cursorTimeoutId);
			delete this._cursorTimeoutId;
		}
	}));

	handlers[handlers.length] = Main.magnifier.connect('active-changed', Lang.bind(Main.overview.viewSelector.appDisplay._views[AppDisplay.Views['ALL']].view, function (magnifier, activate) {
		if (activate)
			this._scrollView.update_fade_effect(0, 0);
		else {
			let fadeOffset = Math.min(this._grid.topPadding, this._grid.bottomPadding);
			this._scrollView.update_fade_effect(fadeOffset, 0);
		}
	}));

	handlers[handlers.length] = Main.magnifier.connect('active-changed', Lang.bind(Main.overview.viewSelector._searchResults, function (magnifier, activate) {
		if (activate)
			this._scrollView.remove_style_class_name('vfade');
		else
			this._scrollView.add_style_class_name('vfade');
	}));

	if (Main.magnifier.isActive()) {
		Main.magnifier.setActive(false);
		Main.magnifier.setActive(true);
	}
}

function _disconnectHandlers() {
	for (var i = 0; i < handlers.length; i++)
		Main.magnifier.disconnect(handlers[i]);
}

function disable() {
	PointerWatcher.PointerWatcher.prototype.addWatch = injections['addWatch'];
	AppDisplay.AllView.prototype.adaptToSize = injections['adaptToSize'];

	if (Main.magnifier.isActive()) {
		Main.magnifier.setActive(false);
		_disconnectHandlers();
		Main.magnifier.setActive(true);
	} else {
		_disconnectHandlers();
	}
}

function init() {}